﻿using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace PoolPattern
{
    class FondPripojeni
    {
        public class Proxy : IPripojeni, IDisposable
        {
            internal bool Vraceno = false;
            internal ZastupcePripojeni Pripojeni;
            private FondPripojeni fond;
            private bool disposed = false;
            public Proxy(ZastupcePripojeni Pripojeni, FondPripojeni Fond)
            {
                this.Pripojeni = Pripojeni;
                fond = Fond;
            }
            ~Proxy()
            {
                Dispose(false);
            }
            public object Dotaz(string Querry)
            {
                if (Vraceno)
                    throw new Exception("Nadále již nesmíte objekt používat");
                return Pripojeni.Dotaz(Querry);
            }
            public void Dispose()
            {
                Dispose(true);
                GC.SuppressFinalize(this);
            }
            protected virtual void Dispose(bool Disposing)
            {
                if (!disposed)
                {
                    fond.VratitPripojeni(this);
                    disposed = true;
                }
            }
        }


        private List<ZastupcePripojeni> SeznamPripojeni = new List<ZastupcePripojeni>();
        private int MaximalniPocetInstanci = 10;
        private int Vytvoreno = 0;

        public Proxy ZiskejPripojeniIhned()
        {
            Proxy Vratit;
            lock (SeznamPripojeni)
            {
                if (SeznamPripojeni.Count == 0)
                {
                    ZastupcePripojeni Pripojeni = new ZastupcePripojeni();
                    Vratit = new Proxy(Pripojeni, this);
                }
                else
                {
                    ZastupcePripojeni Pripojeni = SeznamPripojeni[0];
                    SeznamPripojeni.RemoveAt(0);
                    Vratit = new Proxy(Pripojeni, this);
                }
            }
            return Vratit;
        }
        public Proxy ZiskejPripojeniPockam()
        {
            Proxy Vratit = null;
            while (true)
            {
                lock (SeznamPripojeni)
                {
                    if (SeznamPripojeni.Count == 0 && Vytvoreno < MaximalniPocetInstanci)
                    {
                        ZastupcePripojeni Pripojeni = new ZastupcePripojeni();
                        SeznamPripojeni.Add(Pripojeni);
                        Vytvoreno++;
                    }
                    if (SeznamPripojeni.Count != 0)
                    {
                        ZastupcePripojeni Pripojeni = SeznamPripojeni[0];
                        SeznamPripojeni.RemoveAt(0);
                        Vratit = new Proxy(Pripojeni, this);
                    }
                }
                if (Vratit != null)
                    break;

                Thread.Sleep(0);
            }
            return Vratit;
        }

        public void VratitPripojeni(Proxy ZastupcePripojeni)
        {
            lock (SeznamPripojeni)
            {
                ZastupcePripojeni.Pripojeni.ResetovatStav();
                if (SeznamPripojeni.Count < MaximalniPocetInstanci)
                    SeznamPripojeni.Add(ZastupcePripojeni.Pripojeni);
                ZastupcePripojeni.Pripojeni = null;
                ZastupcePripojeni.Vraceno = true;
            }
        }
    }
}
